[svn-commits] qwell: branch certified-1.8.11 r369839 - in /certified/branches/1.8.11: apps/...

SVN commits to the Digium repositories svn-commits at lists.digium.com
Mon Jul 9 14:06:00 CDT 2012


Author: qwell
Date: Mon Jul  9 14:05:54 2012
New Revision: 369839

URL: http://svnview.digium.com/svn/asterisk?view=rev&rev=369839
Log:
Re-merge changes that were reverted.

------------------------------------------------------------------------
r365395 | qwell | 2012-05-04 16:17:08 -0500 (Fri, 04 May 2012) | 7 lines

Add support for folders in MixMonitor 'm' option.  Backport manager actions.

The manager actions are needed, so MixMonitor can be executed on existing
channels.

(issue DPMA-68)

------------------------------------------------------------------------
r364761 | qwell | 2012-05-01 12:25:14 -0500 (Tue, 01 May 2012) | 6 lines

Remove folder_dir from voicemail snapshots API.

It was both unused (except in tests, where it was fudged) and unnecessary.

(closes issue AST-842)

------------------------------------------------------------------------
r367161 | mmichelson | 2012-05-21 14:05:52 -0500 (Mon, 21 May 2012) | 21 lines

Add "send to voicemail" Digium phone functionality to Asterisk.

This change accommodates two methods by which calls can be directed to
a user's voicemail.

* Incoming calls can be redirected to any user's voicemail.
* Established calls can be blind transferred to any user's voicemail.

Digium phones indicate the desire to direct a call to voicemail by using
a Diversion header with a reason parameter of "send_to_vm".

This patch adds the "send_to_vm" reason as a valid redirecting reason. In
addition, chan_sip.c has been modified to update redirecting information
on the transferred channel by reading a Diversion header on a REFER request.

(closes issue AST-871)
Reported by Malcolm Davenport

Review: https://reviewboard.asterisk.org/r/1925

------------------------------------------------------------------------
r368790 | mjordan | 2012-06-12 08:44:36 -0500 (Tue, 12 Jun 2012) | 18 lines

Fix deadlock in SIP transfers that involve a REFER request

In r367163, "send to voicemail" functionality was added to the SIP channel
driver.  This required updating the party redirecting information for the
channel based on the headers provided in the REFER request.  When the
redirecting party information is updated on the channel, a call to
ast_indicate_data occurs.  Because handle_request_refer still had the sip_pvt
locked, a deadlock could occur between the pbx_thread and the do_monitor thread
servicing the REFER request.

This patch preserves the proper locking order between the channel and the
sip_pvt by ensuring that the sip_pvt is unlocked prior to updating the party
redirecting information on the channel.

(closes issue AST-903)
Reported by: Matt Jordan
patches:
  jira_ast_903_trunk.patch by rmudgett (license 5621)

------------------------------------------------------------------------
r368962 | qwell | 2012-06-14 13:38:48 -0500 (Thu, 14 Jun 2012) | 11 lines

Remove global symbol requirement from app_voicemail.

This uses the existing "function installation" stuff that already existed for
other functions, like getting message counts.

(closes issue AST-807)
(issue AST-901)
(issue AST-908)

Review: https://reviewboard.asterisk.org/r/1965/

------------------------------------------------------------------------
r368964 | qwell | 2012-06-14 14:03:24 -0500 (Thu, 14 Jun 2012) | 8 lines

These functions that were moved need to be static.

Also wrap test functions in a #ifdef.

(issue AST-807)
(issue AST-901)
(issue AST-908)

------------------------------------------------------------------------
r368998 | qwell | 2012-06-15 10:31:43 -0500 (Fri, 15 Jun 2012) | 6 lines

Remove some symbol exports that got missed in the removal of global symbols.

(issue AST-807)
(issue AST-901)
(issue AST-908)

------------------------------------------------------------------------
r369024 | qwell | 2012-06-15 11:29:40 -0500 (Fri, 15 Jun 2012) | 2 lines

Fix voicemail API tests by using the correct argument order for create/destroy.

------------------------------------------------------------------------

Modified:
    certified/branches/1.8.11/apps/app_mixmonitor.c
    certified/branches/1.8.11/apps/app_voicemail.c
    certified/branches/1.8.11/apps/app_voicemail.exports.in
    certified/branches/1.8.11/channels/chan_sip.c
    certified/branches/1.8.11/include/asterisk/app.h
    certified/branches/1.8.11/include/asterisk/app_voicemail.h
    certified/branches/1.8.11/include/asterisk/callerid.h
    certified/branches/1.8.11/main/app.c
    certified/branches/1.8.11/main/callerid.c
    certified/branches/1.8.11/tests/test_voicemail_api.c

Modified: certified/branches/1.8.11/apps/app_mixmonitor.c
URL: http://svnview.digium.com/svn/asterisk/certified/branches/1.8.11/apps/app_mixmonitor.c?view=diff&rev=369839&r1=369838&r2=369839
==============================================================================
--- certified/branches/1.8.11/apps/app_mixmonitor.c (original)
+++ certified/branches/1.8.11/apps/app_mixmonitor.c Mon Jul  9 14:05:54 2012
@@ -98,7 +98,8 @@
 					<option name="m">
 						<argument name="mailbox" required="true" />
 						<para>Create a copy of the recording as a voicemail in each indicated <emphasis>mailbox</emphasis>
-						separated by commas eg. m(1111 at default,2222 at default,...)</para>
+						separated by commas eg. m(1111 at default,2222 at default,...).  Folders can be optionally specified using
+						the syntax: mailbox at context/folder</para>
 						<note><para>The recording will be deleted once all the copies are made.</para></note>
 					</option>
 				</optionlist>
@@ -159,6 +160,54 @@
 			<para>This action may be used to mute a MixMonitor recording.</para>
 		</description>
 	</manager>
+	<manager name="MixMonitor" language="en_US">
+		<synopsis>
+			Record a call and mix the audio during the recording.  Use of StopMixMonitor is required
+			to guarantee the audio file is available for processing during dialplan execution.
+		</synopsis>
+		<syntax>
+			<xi:include xpointer="xpointer(/docs/manager[@name='Login']/syntax/parameter[@name='ActionID'])" />
+			<parameter name="Channel" required="true">
+				<para>Used to specify the channel to record.</para>
+			</parameter>
+			<parameter name="File">
+				<para>Is the name of the file created in the monitor spool directory.
+				Defaults to the same name as the channel (with slashes replaced with dashes).
+				This argument is optional if you specify to record unidirectional audio with
+				either the r(filename) or t(filename) options in the options field. If
+				neither MIXMONITOR_FILENAME or this parameter is set, the mixed stream won't
+				be recorded.</para>
+			</parameter>
+			<parameter name="options">
+				<para>Options that apply to the MixMonitor in the same way as they
+				would apply if invoked from the MixMonitor application. For a list of
+				available options, see the documentation for the mixmonitor application. </para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>This action records the audio on the current channel to the specified file.</para>
+			<variablelist>
+				<variable name="MIXMONITOR_FILENAME">
+					<para>Will contain the filename used to record the mixed stream.</para>
+				</variable>
+			</variablelist>
+		</description>
+	</manager>
+	<manager name="StopMixMonitor" language="en_US">
+		<synopsis>
+			Stop recording a call through MixMonitor, and free the recording's file handle.
+		</synopsis>
+		<syntax>
+			<xi:include xpointer="xpointer(/docs/manager[@name='Login']/syntax/parameter[@name='ActionID'])" />
+			<parameter name="Channel" required="true">
+				<para>The name of the channel monitored.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>This action stops the audio recording that was started with the <literal>MixMonitor</literal>
+			action on the current channel.</para>
+		</description>
+	</manager>
 
  ***/
 
@@ -177,6 +226,7 @@
 struct vm_recipient {
 	char mailbox[AST_MAX_CONTEXT];
 	char context[AST_MAX_EXTENSION];
+	char folder[80];
 	AST_LIST_ENTRY(vm_recipient) list;
 };
 
@@ -310,9 +360,10 @@
  */
 static void add_vm_recipients_from_string(struct mixmonitor *mixmonitor, const char *vm_recipients)
 {
-	/* recipients are in a single string with a format format resembling "mailbox at context,mailbox2 at context2, mailbox3 at context3" */
+	/* recipients are in a single string with a format format resembling "mailbox at context/INBOX,mailbox2 at context2,mailbox3 at context3/Work" */
 	char *cur_mailbox = ast_strdupa(vm_recipients);
 	char *cur_context;
+	char *cur_folder;
 	char *next;
 	int elements_processed = 0;
 
@@ -322,6 +373,12 @@
 			*(next++) = '\0';
 		}
 
+		if ((cur_folder = strchr(cur_mailbox, '/'))) {
+			*(cur_folder++) = '\0';
+		} else {
+			cur_folder = "INBOX";
+		}
+
 		if ((cur_context = strchr(cur_mailbox, '@'))) {
 			*(cur_context++) = '\0';
 		} else {
@@ -329,7 +386,6 @@
 		}
 
 		if (!ast_strlen_zero(cur_mailbox) && !ast_strlen_zero(cur_context)) {
-
 			struct vm_recipient *recipient;
 			if (!(recipient = ast_malloc(sizeof(*recipient)))) {
 				ast_log(LOG_ERROR, "Failed to allocate recipient. Aborting function.\n");
@@ -337,11 +393,11 @@
 			}
 			ast_copy_string(recipient->context, cur_context, sizeof(recipient->context));
 			ast_copy_string(recipient->mailbox, cur_mailbox, sizeof(recipient->mailbox));
+			ast_copy_string(recipient->folder, cur_folder, sizeof(recipient->folder));
 
 			/* Add to list */
 			ast_verb(5, "Adding %s@%s to recipient list\n", recipient->mailbox, recipient->context);
 			AST_LIST_INSERT_HEAD(&mixmonitor->recipient_list, recipient, list);
-
 		} else {
 			ast_log(LOG_ERROR, "Failed to properly parse extension and/or context from element %d of recipient string: %s\n", elements_processed, vm_recipients);
 		}
@@ -410,9 +466,11 @@
 	recording_data.call_priority = mixmonitor->call_priority;
 
 	AST_LIST_TRAVERSE(&mixmonitor->recipient_list, recipient, list) {
-		/* context and mailbox need to be set per recipient */
+		/* context, mailbox, and folder need to be set per recipient */
 		ast_string_field_set(&recording_data, context, recipient->context);
 		ast_string_field_set(&recording_data, mailbox, recipient->mailbox);
+		ast_string_field_set(&recording_data, folder, recipient->folder);
+
 		ast_verb(4, "MixMonitor attempting to send voicemail copy to %s@%s\n", recording_data.mailbox,
 			recording_data.context);
 		ast_app_copy_recording_to_vm(&recording_data);
@@ -904,6 +962,95 @@
 	return AMI_SUCCESS;
 }
 
+static int manager_mixmonitor(struct mansession *s, const struct message *m)
+{
+	struct ast_channel *c = NULL;
+
+	const char *name = astman_get_header(m, "Channel");
+	const char *id = astman_get_header(m, "ActionID");
+	const char *file = astman_get_header(m, "File");
+	const char *options = astman_get_header(m, "Options");
+
+	int res;
+	char args[PATH_MAX] = "";
+	if (ast_strlen_zero(name)) {
+		astman_send_error(s, m, "No channel specified");
+		return AMI_SUCCESS;
+	}
+
+	c = ast_channel_get_by_name(name);
+
+	if (!c) {
+		astman_send_error(s, m, "No such channel");
+		return AMI_SUCCESS;
+	}
+
+	strcpy(args, file);
+	strcat(args, ",");
+	strcat(args, options);
+
+	ast_channel_lock(c);
+	res = mixmonitor_exec(c, args);
+	ast_channel_unlock(c);
+
+	if (res) {
+		astman_send_error(s, m, "Could not start monitoring channel");
+		return AMI_SUCCESS;
+	}
+
+	astman_append(s, "Response: Success\r\n");
+
+	if (!ast_strlen_zero(id)) {
+		astman_append(s, "ActionID: %s\r\n", id);
+	}
+
+	astman_append(s, "\r\n");
+
+	c = ast_channel_unref(c);
+
+	return AMI_SUCCESS;
+}
+
+static int manager_stop_mixmonitor(struct mansession *s, const struct message *m)
+{
+	struct ast_channel *c = NULL;
+
+	const char *name = astman_get_header(m, "Channel");
+	const char *id = astman_get_header(m, "ActionID");
+
+	int res;
+	if (ast_strlen_zero(name)) {
+		astman_send_error(s, m, "No channel specified");
+		return AMI_SUCCESS;
+	}
+
+	c = ast_channel_get_by_name(name);
+
+	if (!c) {
+		astman_send_error(s, m, "No such channel");
+		return AMI_SUCCESS;
+	}
+
+	res = stop_mixmonitor_exec(c, NULL);
+
+	if (res) {
+		astman_send_error(s, m, "Could not stop monitoring channel");
+		return AMI_SUCCESS;
+	}
+
+	astman_append(s, "Response: Success\r\n");
+
+	if (!ast_strlen_zero(id)) {
+		astman_append(s, "ActionID: %s\r\n", id);
+	}
+
+	astman_append(s, "\r\n");
+
+	c = ast_channel_unref(c);
+
+	return AMI_SUCCESS;
+}
+
 static struct ast_cli_entry cli_mixmonitor[] = {
 	AST_CLI_DEFINE(handle_cli_mixmonitor, "Execute a MixMonitor command")
 };
@@ -916,6 +1063,8 @@
 	res = ast_unregister_application(stop_app);
 	res |= ast_unregister_application(app);
 	res |= ast_manager_unregister("MixMonitorMute");
+	res |= ast_manager_unregister("MixMonitor");
+	res |= ast_manager_unregister("StopMixMonitor");
 	
 	return res;
 }
@@ -928,6 +1077,8 @@
 	res = ast_register_application_xml(app, mixmonitor_exec);
 	res |= ast_register_application_xml(stop_app, stop_mixmonitor_exec);
 	res |= ast_manager_register_xml("MixMonitorMute", 0, manager_mute_mixmonitor);
+	res |= ast_manager_register_xml("MixMonitor", 0, manager_mixmonitor);
+	res |= ast_manager_register_xml("StopMixMonitor", 0, manager_stop_mixmonitor);
 
 	return res;
 }

Modified: certified/branches/1.8.11/apps/app_voicemail.c
URL: http://svnview.digium.com/svn/asterisk/certified/branches/1.8.11/apps/app_voicemail.c?view=diff&rev=369839&r1=369838&r2=369839
==============================================================================
--- certified/branches/1.8.11/apps/app_voicemail.exports.in (original)
+++ certified/branches/1.8.11/apps/app_voicemail.exports.in Mon Jul  9 14:05:54 2012
@@ -15,15 +15,6 @@
 		LINKER_SYMBOL_PREFIXmm_notify;
 		LINKER_SYMBOL_PREFIXmm_searched;
 		LINKER_SYMBOL_PREFIXmm_status;
-		LINKER_SYMBOL_PREFIXast_vm_mailbox_snapshot_create;
-		LINKER_SYMBOL_PREFIXast_vm_mailbox_snapshot_destroy;
-		LINKER_SYMBOL_PREFIXast_vm_msg_move;
-		LINKER_SYMBOL_PREFIXast_vm_msg_remove;
-		LINKER_SYMBOL_PREFIXast_vm_msg_forward;
-		LINKER_SYMBOL_PREFIXast_vm_index_to_foldername;
-		LINKER_SYMBOL_PREFIXast_vm_msg_play;
-		LINKER_SYMBOL_PREFIXast_vm_test_create_user;
-		LINKER_SYMBOL_PREFIXast_vm_test_destroy_user;
 	local:
 		*;
 };

Modified: certified/branches/1.8.11/channels/chan_sip.c
URL: http://svnview.digium.com/svn/asterisk/certified/branches/1.8.11/channels/chan_sip.c?view=diff&rev=369839&r1=369838&r2=369839
==============================================================================
--- certified/branches/1.8.11/channels/chan_sip.c (original)
+++ certified/branches/1.8.11/channels/chan_sip.c Mon Jul  9 14:05:54 2012
@@ -683,7 +683,8 @@
 	{ AST_REDIRECTING_REASON_FOLLOW_ME, "follow-me" },
 	{ AST_REDIRECTING_REASON_OUT_OF_ORDER, "out-of-service" },
 	{ AST_REDIRECTING_REASON_AWAY, "away" },
-	{ AST_REDIRECTING_REASON_CALL_FWD_DTE, "unknown"}
+	{ AST_REDIRECTING_REASON_CALL_FWD_DTE, "unknown"},
+	{ AST_REDIRECTING_REASON_SEND_TO_VM, "send_to_vm"},
 };
 
 
@@ -15390,7 +15391,7 @@
 			}
 			/* Remove enclosing double-quotes */
 			if (*reason_param == '"')
-				ast_strip_quoted(reason_param, "\"", "\"");
+				reason_param = ast_strip_quoted(reason_param, "\"", "\"");
 			if (!ast_strlen_zero(reason_param)) {
 				sip_set_redirstr(p, reason_param);
 				if (p->owner) {
@@ -23673,6 +23674,8 @@
 	int localtransfer = 0;
 	int attendedtransfer = 0;
 	int res = 0;
+	struct ast_party_redirecting redirecting;
+	struct ast_set_party_redirecting update_redirecting;
 
 	if (req->debug) {
 		ast_verbose("Call %s got a SIP call transfer from %s: (REFER)!\n",
@@ -23977,13 +23980,25 @@
 	}
 	ast_set_flag(&p->flags[0], SIP_DEFER_BYE_ON_TRANSFER);	/* Delay hangup */
 
-	/* Do not hold the pvt lock during the indicate and async_goto. Those functions
-	 * lock channels which will invalidate locking order if the pvt lock is held.*/
+	/* When a call is transferred to voicemail from a Digium phone, there may be
+	 * a Diversion header present in the REFER with an appropriate reason parameter
+	 * set. We need to update the redirecting information appropriately.
+	 */
+	ast_party_redirecting_init(&redirecting);
+	memset(&update_redirecting, 0, sizeof(update_redirecting));
+	change_redirecting_information(p, req, &redirecting, &update_redirecting, FALSE);
+
+	/* Do not hold the pvt lock during a call that causes an indicate or an async_goto.
+	 * Those functions lock channels which will invalidate locking order if the pvt lock
+	 * is held.*/
+	sip_pvt_unlock(p);
+	ast_channel_update_redirecting(current.chan2, &redirecting, &update_redirecting);
+	ast_party_redirecting_free(&redirecting);
+
 	/* For blind transfers, move the call to the new extensions. For attended transfers on multiple
 	 * servers - generate an INVITE with Replaces. Either way, let the dial plan decided
 	 * indicate before masquerade so the indication actually makes it to the real channel
 	 * when using local channels with MOH passthru */
-	sip_pvt_unlock(p);
 	ast_indicate(current.chan2, AST_CONTROL_UNHOLD);
 	res = ast_async_goto(current.chan2, refer_to_context, refer_to, 1);
 

Modified: certified/branches/1.8.11/include/asterisk/app.h
URL: http://svnview.digium.com/svn/asterisk/certified/branches/1.8.11/include/asterisk/app.h?view=diff&rev=369839&r1=369838&r2=369839
==============================================================================
--- certified/branches/1.8.11/include/asterisk/app.h (original)
+++ certified/branches/1.8.11/include/asterisk/app.h Mon Jul  9 14:05:54 2012
@@ -27,6 +27,7 @@
 #include "asterisk/strings.h"
 #include "asterisk/threadstorage.h"
 #include "asterisk/file.h"
+#include "asterisk/linkedlists.h"
 
 struct ast_flags64;
 
@@ -157,13 +158,52 @@
 int ast_app_run_macro(struct ast_channel *autoservice_chan, struct ast_channel 
 		*macro_chan, const char * const macro_name, const char * const macro_args);
 
+enum ast_vm_snapshot_sort_val {
+	AST_VM_SNAPSHOT_SORT_BY_ID = 0,
+	AST_VM_SNAPSHOT_SORT_BY_TIME,
+};
+
+struct ast_vm_msg_snapshot {
+	AST_DECLARE_STRING_FIELDS(
+		AST_STRING_FIELD(msg_id);
+		AST_STRING_FIELD(callerid);
+		AST_STRING_FIELD(callerchan);
+		AST_STRING_FIELD(exten);
+		AST_STRING_FIELD(origdate);
+		AST_STRING_FIELD(origtime);
+		AST_STRING_FIELD(duration);
+		AST_STRING_FIELD(folder_name);
+		AST_STRING_FIELD(flag);
+	);
+	unsigned int msg_number;
+
+	AST_LIST_ENTRY(ast_vm_msg_snapshot) msg;
+};
+
+struct ast_vm_mailbox_snapshot {
+	int total_msg_num;
+	int folders;
+	/* Things are not quite as they seem here.  This points to an allocated array of lists. */
+	AST_LIST_HEAD_NOLOCK(, ast_vm_msg_snapshot) *snapshots;
+};
+
+/*!
+ * \brief Voicemail playback callback function definition
+ *
+ * \param channel to play the file back on.
+ * \param location of file on disk
+ * \param duration of file in seconds. This will be zero if msg is very short or
+ * has an unknown duration.
+ */
+typedef void (ast_vm_msg_play_cb)(struct ast_channel *chan, const char *playfile, int duration);
+
 /*!
  * \brief Set voicemail function callbacks
  * \param[in] has_voicemail_func set function pointer
+ * \param[in] inboxcount_func set function pointer
  * \param[in] inboxcount2_func set function pointer
+ * \param[in] messagecount_func set function pointer
  * \param[in] sayname_func set function pointer
- * \param[in] inboxcount_func set function pointer
- * \param[in] messagecount_func set function pointer
  * \version 1.6.1 Added inboxcount2_func, sayname_func
  */
 void ast_install_vm_functions(int (*has_voicemail_func)(const char *mailbox, const char *folder),
@@ -171,9 +211,51 @@
 			      int (*inboxcount2_func)(const char *mailbox, int *urgentmsgs, int *newmsgs, int *oldmsgs),
 			      int (*messagecount_func)(const char *context, const char *mailbox, const char *folder),
 			      int (*sayname_func)(struct ast_channel *chan, const char *mailbox, const char *context),
-			      int (*copy_recording_to_vm_func)(struct ast_vm_recording_data *vm_rec_data));
+			      int (*copy_recording_to_vm_func)(struct ast_vm_recording_data *vm_rec_data),
+			      const char *vm_index_to_foldername(int id),
+			      struct ast_vm_mailbox_snapshot *(*vm_mailbox_snapshot_create)(const char *mailbox,
+				const char *context,
+				const char *folder,
+				int descending,
+				enum ast_vm_snapshot_sort_val sort_val,
+				int combine_INBOX_and_OLD),
+			      struct ast_vm_mailbox_snapshot *(*vm_mailbox_snapshot_destroy)(struct ast_vm_mailbox_snapshot *mailbox_snapshot),
+			      int (*vm_msg_move)(const char *mailbox,
+				const char *context,
+				size_t num_msgs,
+				const char *oldfolder,
+				int *old_msg_ids,
+				const char *newfolder,
+				int *new_msg_ids),
+			      int (*vm_msg_remove)(const char *mailbox,
+				const char *context,
+				size_t num_msgs,
+				const char *folder,
+				int *msgs),
+			      int (*vm_msg_forward)(const char *from_mailbox,
+				const char *from_context,
+				const char *from_folder,
+				const char *to_mailbox,
+				const char *to_context,
+				const char *to_folder,
+				size_t num_msgs,
+				int *msg_ids,
+				int delete_old),
+			      int (*vm_msg_play)(struct ast_channel *chan,
+				const char *mailbox,
+				const char *context,
+				const char *folder,
+				const char *msg_num,
+				ast_vm_msg_play_cb cb));
 
 void ast_uninstall_vm_functions(void);
+
+#ifdef TEST_FRAMEWORK
+void ast_install_vm_test_functions(int (*vm_test_destroy_user)(const char *context, const char *mailbox),
+				   int (*vm_test_create_user)(const char *context, const char *mailbox));
+
+void ast_uninstall_vm_test_functions(void);
+#endif
 
 /*!
  * \brief
@@ -237,6 +319,133 @@
  * \return Number of messages in the given context, mailbox, and folder.  If folder is NULL, folder "INBOX" is assumed.  If folder is "INBOX", includes number of messages in the "Urgent" folder.
  */
 int ast_app_messagecount(const char *context, const char *mailbox, const char *folder);
+
+/*!
+ * \brief Return name of folder, given an id
+ * \param[in] id Folder id
+ * \return Name of folder
+ */
+const char *ast_vm_index_to_foldername(int id);
+
+/*
+ * \brief Create a snapshot of a mailbox which contains information about every msg.
+ *
+ * \param mailbox, the mailbox to look for
+ * \param context, the context to look for the mailbox in
+ * \param folder, OPTIONAL.  When not NULL only msgs from the specified folder will be included.
+ * \param desending, list the msgs in descending order rather than ascending order.
+ * \param combine_INBOX_and_OLD, When this argument is set, The OLD folder will be represented
+ *        in the INBOX folder of the snapshot. This allows the snapshot to represent the
+ *        OLD and INBOX messages in sorted order merged together.
+ *
+ * \retval snapshot on success
+ * \retval NULL on failure
+ */
+struct ast_vm_mailbox_snapshot *ast_vm_mailbox_snapshot_create(const char *mailbox,
+	const char *context,
+	const char *folder,
+	int descending,
+	enum ast_vm_snapshot_sort_val sort_val,
+	int combine_INBOX_and_OLD);
+
+/*
+ * \brief destroy a snapshot
+ *
+ * \param mailbox_snapshot The snapshot to destroy.
+ * \retval NULL
+ */
+struct ast_vm_mailbox_snapshot *ast_vm_mailbox_snapshot_destroy(struct ast_vm_mailbox_snapshot *mailbox_snapshot);
+
+/*!
+ * \brief Move messages from one folder to another
+ *
+ * \param mailbox The mailbox to which the folders belong
+ * \param context The voicemail context for the mailbox
+ * \param num_msgs The number of messages to move
+ * \param oldfolder The folder from where messages should be moved
+ * \param old_msg_nums The message IDs of the messages to move
+ * \param newfolder The folder to which messages should be moved
+ * \param new_msg_ids[out] An array of message IDs for the messages as they are in the
+ * new folder. This array must be num_msgs sized.
+ *
+ * \retval -1 Failure
+ * \retval 0 Success
+ */
+int ast_vm_msg_move(const char *mailbox,
+	const char *context,
+	size_t num_msgs,
+	const char *oldfolder,
+	int *old_msg_ids,
+	const char *newfolder,
+	int *new_msg_ids);
+
+/*!
+ * \brief Remove/delete messages from a mailbox folder.
+ *
+ * \param mailbox The mailbox from which to delete messages
+ * \param context The voicemail context for the mailbox
+ * \param num_msgs The number of messages to delete
+ * \param folder The folder from which to remove messages
+ * \param msgs The message IDs of the messages to delete
+ *
+ * \retval -1 Failure
+ * \retval 0 Success
+ */
+int ast_vm_msg_remove(const char *mailbox,
+	const char *context,
+	size_t num_msgs,
+	const char *folder,
+	int *msgs);
+
+/*!
+ * \brief forward a message from one mailbox to another.
+ *
+ * \brief from_mailbox The original mailbox the message is being forwarded from
+ * \brief from_context The voicemail context of the from_mailbox
+ * \brief from_folder The folder from which the message is being forwarded
+ * \brief to_mailbox The mailbox to forward the message to
+ * \brief to_context The voicemail context of the to_mailbox
+ * \brief to_folder The folder to which the message is being forwarded
+ * \brief num_msgs The number of messages being forwarded
+ * \brief msg_ids The message IDs of the messages in from_mailbox to forward
+ * \brief delete_old If non-zero, the forwarded messages are also deleted from from_mailbox.
+ * Otherwise, the messages will remain in the from_mailbox.
+ *
+ * \retval -1 Failure
+ * \retval 0 Success
+ */
+int ast_vm_msg_forward(const char *from_mailbox,
+	const char *from_context,
+	const char *from_folder,
+	const char *to_mailbox,
+	const char *to_context,
+	const char *to_folder,
+	size_t num_msgs,
+	int *msg_ids,
+	int delete_old);
+
+/*!
+ * \brief Play a voicemail msg back on a channel.
+ *
+ * \param mailbox msg is in.
+ * \param context of mailbox.
+ * \param voicemail folder to look in.
+ * \param message number in the voicemailbox to playback to the channel.
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_vm_msg_play(struct ast_channel *chan,
+	const char *mailbox,
+	const char *context,
+	const char *folder,
+	const char *msg_num,
+	ast_vm_msg_play_cb cb);
+
+#ifdef TEST_FRAMEWORK
+int ast_vm_test_destroy_user(const char *context, const char *mailbox);
+int ast_vm_test_create_user(const char *context, const char *mailbox);
+#endif
 
 /*! \brief Safely spawn an external program while closing file descriptors
 	\note This replaces the \b system call in all Asterisk modules

Modified: certified/branches/1.8.11/include/asterisk/app_voicemail.h
URL: http://svnview.digium.com/svn/asterisk/certified/branches/1.8.11/include/asterisk/app_voicemail.h?view=diff&rev=369839&r1=369838&r2=369839
==============================================================================
--- certified/branches/1.8.11/include/asterisk/app_voicemail.h (original)
+++ certified/branches/1.8.11/include/asterisk/app_voicemail.h Mon Jul  9 14:05:54 2012
@@ -1,214 +1,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2011, Digium, Inc.
- *
- * David Vossel <dvossel at digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- * \brief Voice Mail API
- * \author David Vossel <dvossel at digium.com>
- */
-
-#ifndef _ASTERISK_VM_H
-#define _ASTERISK_VM_H
-
-#include "asterisk/stringfields.h"
-#include "asterisk/linkedlists.h"
-
-#define AST_VM_FOLDER_NUMBER 12
-
-enum ast_vm_snapshot_sort_val {
-	AST_VM_SNAPSHOT_SORT_BY_ID = 0,
-	AST_VM_SNAPSHOT_SORT_BY_TIME,
-};
-
-struct ast_vm_msg_snapshot {
-	AST_DECLARE_STRING_FIELDS(
-		AST_STRING_FIELD(msg_id);
-		AST_STRING_FIELD(callerid);
-		AST_STRING_FIELD(callerchan);
-		AST_STRING_FIELD(exten);
-		AST_STRING_FIELD(origdate);
-		AST_STRING_FIELD(origtime);
-		AST_STRING_FIELD(duration);
-		AST_STRING_FIELD(folder_name);
-		AST_STRING_FIELD(folder_dir);
-		AST_STRING_FIELD(flag);
-	);
-	unsigned int msg_number;
-
-	AST_LIST_ENTRY(ast_vm_msg_snapshot) msg;
-};
-
-struct ast_vm_mailbox_snapshot {
-	int total_msg_num;
-	AST_LIST_HEAD_NOLOCK(, ast_vm_msg_snapshot) snapshots[AST_VM_FOLDER_NUMBER];
-};
-
-/*
- * \brief Create a snapshot of a mailbox which contains information about every msg.
- *
- * \param mailbox, the mailbox to look for
- * \param context, the context to look for the mailbox in
- * \param folder, OPTIONAL.  When not NULL only msgs from the specified folder will be included.
- * \param desending, list the msgs in descending order rather than ascending order.
- * \param combine_INBOX_and_OLD, When this argument is set, The OLD folder will be represented
- *        in the INBOX folder of the snapshot. This allows the snapshot to represent the
- *        OLD and INBOX messages in sorted order merged together.
- *
- * \retval snapshot on success
- * \retval NULL on failure
- */
-struct ast_vm_mailbox_snapshot *ast_vm_mailbox_snapshot_create(const char *mailbox,
-	const char *context,
-	const char *folder,
-	int descending,
-	enum ast_vm_snapshot_sort_val sort_val,
-	int combine_INBOX_and_OLD);
-
-/*
- * \brief destroy a snapshot
- *
- * \param mailbox_snapshot The snapshot to destroy.
- * \retval NULL
- */
-struct ast_vm_mailbox_snapshot *ast_vm_mailbox_snapshot_destroy(struct ast_vm_mailbox_snapshot *mailbox_snapshot);
-
-/*!
- * \brief Move messages from one folder to another
- *

[... 537 lines stripped ...]



More information about the svn-commits mailing list
--- certified/branches/1.8.11/apps/app_voicemail.c (original)
+++ certified/branches/1.8.11/apps/app_voicemail.c Mon Jul  9 14:05:54 2012
@@ -113,7 +113,6 @@
 #include "asterisk/module.h"
 #include "asterisk/adsi.h"
 #include "asterisk/app.h"
-#include "asterisk/app_voicemail.h"
 #include "asterisk/manager.h"
 #include "asterisk/dsp.h"
 #include "asterisk/localtime.h"
@@ -969,9 +968,21 @@
 static int is_valid_dtmf(const char *key);
 static void read_password_from_file(const char *secretfn, char *password, int passwordlen);
 static int write_password_to_file(const char *secretfn, const char *password);
-struct ast_str *vm_mailbox_snapshot_str(const char *mailbox, const char *context);
 static const char *substitute_escapes(const char *value);
 static void free_user(struct ast_vm_user *vmu);
+
+static struct ast_vm_mailbox_snapshot *vm_mailbox_snapshot_create(const char *mailbox, const char *context, const char *folder, int descending, enum ast_vm_snapshot_sort_val sort_val, int combine_INBOX_and_OLD);
+static struct ast_vm_mailbox_snapshot *vm_mailbox_snapshot_destroy(struct ast_vm_mailbox_snapshot *mailbox_snapshot);
+
+static int vm_msg_forward(const char *from_mailbox, const char *from_context, const char *from_folder, const char *to_mailbox, const char *to_context, const char *to_folder, size_t num_msgs, int *msg_ids, int delete_old);
+static int vm_msg_move(const char *mailbox, const char *context, size_t num_msgs, const char *oldfolder, int *old_msg_nums, const char *newfolder, int *new_msg_nums);
+static int vm_msg_remove(const char *mailbox, const char *context, size_t num_msgs, const char *folder, int *msgs);
+static int vm_msg_play(struct ast_channel *chan, const char *mailbox, const char *context, const char *folder, const char *msg_num, ast_vm_msg_play_cb cb);
+
+#ifdef TEST_FRAMEWORK
+static int vm_test_destroy_user(const char *context, const char *mailbox);
+static int vm_test_create_user(const char *context, const char *mailbox);
+#endif
 
 struct ao2_container *inprocess_container;
 
@@ -1756,6 +1767,12 @@
 	return (id >= 0 && id < ARRAY_LEN(mailbox_folders)) ? mailbox_folders[id] : "Unknown";
 }
 
+static const char *vm_index_to_foldername(int id)
+{
+	return mbox(NULL, id);
+}
+
+
 static int get_folder_by_name(const char *name)
 {
 	size_t i;
@@ -5757,7 +5774,7 @@
 	/* At this point, the actual creation of a voicemail message should be finished.
 	 * Now we just need to copy the files being recorded into the receiving folder. */
 
-	create_dirpath(dir, sizeof(dir), recipient->context, recipient->mailbox, "INBOX");
+	create_dirpath(dir, sizeof(dir), recipient->context, recipient->mailbox, recdata->folder);
 
 #ifdef IMAP_STORAGE
 	/* make recipient info into an inboxcount friendly string */
@@ -10276,7 +10293,7 @@
 	}
 
 	/* Iterate through every folder, find the msg, and play it */
-	for (i = 0; i < AST_VM_FOLDER_NUMBER && !played; i++) {
+	for (i = 0; i < ARRAY_LEN(mailbox_folders) && !played; i++) {
 		ast_copy_string(vms.username, mailbox, sizeof(vms.username));
 		vms.lastmsg = -1;
 
@@ -13539,6 +13556,9 @@
 #endif
 	ast_cli_unregister_multiple(cli_voicemail, ARRAY_LEN(cli_voicemail));
 	ast_uninstall_vm_functions();
+#ifdef TEST_FRAMEWORK
+	ast_uninstall_vm_test_functions();
+#endif
 	ao2_ref(inprocess_container, -1);
 
 	if (poll_thread != AST_PTHREADT_NULL)
@@ -13595,7 +13615,15 @@
 	ast_cli_register_multiple(cli_voicemail, ARRAY_LEN(cli_voicemail));
 	ast_data_register_multiple(vm_data_providers, ARRAY_LEN(vm_data_providers));
 
-	ast_install_vm_functions(has_voicemail, inboxcount, inboxcount2, messagecount, sayname, msg_create_from_file);
+	ast_install_vm_functions(has_voicemail, inboxcount, inboxcount2, messagecount, sayname, msg_create_from_file,
+				 vm_index_to_foldername,
+				 vm_mailbox_snapshot_create, vm_mailbox_snapshot_destroy,
+				 vm_msg_move, vm_msg_remove, vm_msg_forward, vm_msg_play);
+
+#ifdef TEST_FRAMEWORK
+	ast_install_vm_test_functions(vm_test_create_user, vm_test_destroy_user);
+#endif
+
 	ast_realtime_require_field("voicemail", "uniqueid", RQ_UINTEGER3, 11, "password", RQ_CHAR, 10, SENTINEL);
 	ast_realtime_require_field("voicemail_data", "filename", RQ_CHAR, 30, "duration", RQ_UINTEGER3, 5, SENTINEL);
 
@@ -14130,7 +14158,7 @@
 
 #ifdef TEST_FRAMEWORK
 
-int ast_vm_test_destroy_user(const char *context, const char *mailbox)
+static int vm_test_destroy_user(const char *context, const char *mailbox)
 {
 	struct ast_vm_user *vmu;
 
@@ -14148,7 +14176,7 @@
 	return 0;
 }
 
-int ast_vm_test_create_user(const char *context, const char *mailbox)
+static int vm_test_create_user(const char *context, const char *mailbox)
 {
 	struct ast_vm_user *vmu;
 
@@ -14227,9 +14255,6 @@
 		if ((value = ast_variable_retrieve(msg_cfg, "message", "duration"))) {
 			ast_string_field_set(msg_snapshot, duration, value);
 		}
-		if ((value = ast_variable_retrieve(msg_cfg, "message", "folder_dir"))) {
-			ast_string_field_set(msg_snapshot, folder_dir, value);
-		}
 		if ((value = ast_variable_retrieve(msg_cfg, "message", "flag"))) {
 			ast_string_field_set(msg_snapshot, flag, value);
 		}
@@ -14278,7 +14303,7 @@
 	return 0;
 }
 
-struct ast_vm_mailbox_snapshot *ast_vm_mailbox_snapshot_create(const char *mailbox,
+static struct ast_vm_mailbox_snapshot *vm_mailbox_snapshot_create(const char *mailbox,
 	const char *context,
 	const char *folder,
 	int descending,
@@ -14304,7 +14329,7 @@
 
 	if (!(ast_strlen_zero(folder))) {
 		/* find the folder index */
-		for (i = 0; i < AST_VM_FOLDER_NUMBER; i++) {
+		for (i = 0; i < ARRAY_LEN(mailbox_folders); i++) {
 			if (!strcasecmp(mailbox_folders[i], folder)) {
 				this_index_only = i;
 				break;
@@ -14324,7 +14349,14 @@
 		return NULL;
 	}
 
-	for (i = 0; i < AST_VM_FOLDER_NUMBER; i++) {
+	if (!(mailbox_snapshot->snapshots = ast_calloc(ARRAY_LEN(mailbox_folders), sizeof(*mailbox_snapshot->snapshots)))) {
+		ast_free(mailbox_snapshot);
+		return NULL;
+	}
+
+	mailbox_snapshot->folders = ARRAY_LEN(mailbox_folders);
+
+	for (i = 0; i < mailbox_snapshot->folders; i++) {
 		int combining_old = 0;
 		if ((i == old_index) && (combine_INBOX_and_OLD)) {
 			combining_old = 1;
@@ -14379,63 +14411,26 @@
 	return mailbox_snapshot;
 }
 
-struct ast_vm_mailbox_snapshot *ast_vm_mailbox_snapshot_destroy(struct ast_vm_mailbox_snapshot *mailbox_snapshot)
+static struct ast_vm_mailbox_snapshot *vm_mailbox_snapshot_destroy(struct ast_vm_mailbox_snapshot *mailbox_snapshot)
 {
 	int i;
 	struct ast_vm_msg_snapshot *msg_snapshot;
 
-	for (i = 0; i < AST_VM_FOLDER_NUMBER; i++) {
+	for (i = 0; i < mailbox_snapshot->folders; i++) {
 		while ((msg_snapshot = AST_LIST_REMOVE_HEAD(&mailbox_snapshot->snapshots[i], msg))) {
 			msg_snapshot = vm_msg_snapshot_destroy(msg_snapshot);
 		}
 	}
+	ast_free(mailbox_snapshot->snapshots);
 	ast_free(mailbox_snapshot);
 	return NULL;
-}
-
-struct ast_str *vm_mailbox_snapshot_str(const char *mailbox, const char *context)
-{
-	struct ast_vm_mailbox_snapshot *mailbox_snapshot = ast_vm_mailbox_snapshot_create(mailbox, context, NULL, 0, AST_VM_SNAPSHOT_SORT_BY_ID, 0);
-	struct ast_vm_msg_snapshot *msg_snapshot;
-	int i;
-	struct ast_str *str;
-
-	if (!mailbox_snapshot) {
-		return NULL;
-	}
-
-	if (!(str = ast_str_create(512))) {
-		return NULL;
-		mailbox_snapshot = ast_vm_mailbox_snapshot_destroy(mailbox_snapshot);
-	}
-
-	for (i = 0; i < AST_VM_FOLDER_NUMBER; i++) {
-		ast_str_append(&str, 0, "FOLDER: %s\n", mailbox_folders[i]);
-		AST_LIST_TRAVERSE(&mailbox_snapshot->snapshots[i], msg_snapshot, msg) {
-			ast_str_append(&str, 0, "MSG Number:   %d\n", msg_snapshot->msg_number);
-			ast_str_append(&str, 0, "MSG ID:       %s\n", msg_snapshot->msg_id);
-			ast_str_append(&str, 0, "CALLER ID:    %s\n", msg_snapshot->callerid);
-			ast_str_append(&str, 0, "CALLER CHAN:  %s\n", msg_snapshot->callerchan);
-			ast_str_append(&str, 0, "CALLER EXTEN: %s\n", msg_snapshot->exten);
-			ast_str_append(&str, 0, "DATE:         %s\n", msg_snapshot->origdate);
-			ast_str_append(&str, 0, "TIME:         %s\n", msg_snapshot->origtime);
-			ast_str_append(&str, 0, "DURATION:     %s\n", msg_snapshot->duration);
-			ast_str_append(&str, 0, "FOLDER NAME:  %s\n", msg_snapshot->folder_name);
-			ast_str_append(&str, 0, "FOLDER DIR:   %s\n", msg_snapshot->folder_dir);
-			ast_str_append(&str, 0, "FLAG:         %s\n", msg_snapshot->folder_dir);
-			ast_str_append(&str, 0, "\n");
-		}
-	}
-
-	mailbox_snapshot = ast_vm_mailbox_snapshot_destroy(mailbox_snapshot);
-	return str;
 }
 
 /*!
  * \brief common bounds checking and existence check for Voicemail API functions.
  *
  * \details
  * This is called by ast_vm_msg_move, ast_vm_msg_remove, and ast_vm_msg_forward to
  * ensure that data passed in are valid. This tests the following:
  *
  * 1. No negative indexes are given.
@@ -14487,7 +14482,7 @@
 	queue_mwi_event(ext_context, urgent, new, old);
 }
 
-int ast_vm_msg_forward(const char *from_mailbox,
+static int vm_msg_forward(const char *from_mailbox,
 	const char *from_context,
 	const char *from_folder,
 	const char *to_mailbox,
@@ -14624,7 +14619,7 @@
 	return res;
 }
 
-int ast_vm_msg_move(const char *mailbox,
+static int vm_msg_move(const char *mailbox,
 	const char *context,
 	size_t num_msgs,
 	const char *oldfolder,
@@ -14719,7 +14714,7 @@
 	return res;
 }
 
-int ast_vm_msg_remove(const char *mailbox,
+static int vm_msg_remove(const char *mailbox,
 	const char *context,
 	size_t num_msgs,
 	const char *folder,
@@ -14813,15 +14808,7 @@
 	return res;
 }
 
-const char *ast_vm_index_to_foldername(unsigned int index)
-{
-	if (index >= AST_VM_FOLDER_NUMBER) {
-		return "";
-	}
-	return mailbox_folders[index];
-}
-
-int ast_vm_msg_play(struct ast_channel *chan,
+static int vm_msg_play(struct ast_channel *chan,
 	const char *mailbox,
 	const char *context,
 	const char *folder,
@@ -14939,7 +14926,7 @@
  * AST_MODULE_INFO(, , "Comedian Mail (Voicemail System)"
  */
 
-AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, tdesc,
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, tdesc,
 		.load = load_module,
 		.unload = unload_module,
 		.reload = reload,
@@ -14379,63 +14411,26 @@
  *
  * \details
- * This is called by ast_vm_msg_move, ast_vm_msg_remove, and ast_vm_msg_forward to
+ * This is called by vm_msg_move, vm_msg_remove, and vm_msg_forward to
  * ensure that data passed in are valid. This tests the following:
  *

Modified: certified/branches/1.8.11/apps/app_voicemail.exports.in
URL: http://svnview.digium.com/svn/asterisk/certified/branches/1.8.11/apps/app_voicemail.exports.in?view=diff&rev=369839&r1=369838&r2=369839
==============================================================================